/*
  Rev 1 of Wifi_IR_Remote_ESP8266

  Load this sketch to the Adafruit Feather ESP8266 module (Other ESP8266 modules can also be used)
  This code uses Arduino ESP8266 is V2.3.0.  Other versions may not work as expected

  This code drives ledPin to indicate when connected
  Steady led == OK, Have Network connection
  Slow Flash == Have remote connection.
  Fast Flash == Connection or msg problem.

*/

/**
   Wifi_IR_Remote_ESP8266
   https://www.forward.com.au/pfod/HomeAutomation/Wifi_IR_Remote/index.html

   (c)2017 Forward Computing and Control Pty. Ltd.
   This code may be freely used for both private and commerical use.
   Provide this copyright is maintained.
*/


//#define DEBUG

#include <EEPROM.h>
// Download pfodESP8266BufferedClient library from http://www.forward.com.au/pfod/pfodParserLibraries/index.html
#include <ESP8266WiFi.h>
#include <pfodESP8266Utils.h>
#include <pfodESP8266BufferedClient.h>
#include "cmdDefines.h"
#include "WifiWebConfig.h" // include bufferedClient and server and client vars from WifiWebConfig.ino
#include "ESP8266_ledFlasher.h" // include vars for Led flasher

// Download pfodParser library from http://www.forward.com.au/pfod/pfodParserLibraries/index.html
#include <pfodSecurity.h>  // V2.31 or higher
int swap01(int); // method prototype for slider end swaps

pfodSecurity parser("V1"); // create a parser with menu version string to handle the pfod messages
// not pfodSecurity implements connection timeout default 15sec.  pfodApp sends keep alives every 5sec for Wifi connections
//pfodESP8266BufferedClient bufferedClient;  // bufferedClient defined in WifiWebConfig

pfodDwgs dwgs(&parser);  // drawing support
// Commands for drawing controls only need to unique per drawing, you can re-use cmds 'a' in another drawing.

// this filter  i.e.  .filter(BUTTON_REPEATS_FILTER) will make button repeat after long press and stop on release (UP)
// e.g.  for Left Arrow use   dwgs.touchZone().cmd(pvr_leftCmd_A).filter(BUTTON_REPEATS_FILTER).centered().size(6, 6).send();
// default for touchZone with no filter is TOUCH i.e. just send cmd once for each button touch and block until finger removed from screen
#define BUTTON_REPEATS_FILTER (dwgs.DOWN+dwgs.PRESS+dwgs.UP)

// add your pfod Password here for 128bit security
// eg "b0Ux9akSiwKkwCtcnjTnpWp" but generate your own key, "" means no pfod password
#define pfodSecurityCode ""
// see http://www.forward.com.au/pfod/ArduinoWiFi_simple_pfodDevice/index.html for more information and an example
// and QR image key generator.

boolean alreadyConnected = false; // whether or not the client was connected previously


void initVars() {
  initLedFlasher(); // sets ledOn true
}


// the setup routine runs once on reset:
void setup() {
  initVars();
  if (configSetup()) {
    return; // in config mode
  }
  // else have connected to network
  // and started server on configured port
  parser.setIdleTimeout(getConnectionTimeout()/1000); // set pfodSecurity timeout to the one configured via webpage otherwise default is 15sec
  // note getConnectionTimeout() returns mS while setIdleTimeout arg is Sec.
  // <<<<<<<<< Your extra setup code goes here

}

// the loop routine runs over and over again forever:
void loop() {
  if (configLoop()) {
    return; // skip the rest while in config mode
  }
  if (!networkConnected) {
    return; // nothing to do if not connected just flash
  }
  if (!client) { // see if a client is available
    client = server.available(); // evaluates to false if no connection
  } else {
    // have client
    if (!client.connected()) {
      if (alreadyConnected) {
        // client closed so clean up
        closeConnection(parser.getPfodAppStream());
      }
    } else {
      // have connected client
      if (!alreadyConnected) {
        parser.connect(bufferedClient.connect(&client), F(pfodSecurityCode)); // sets new io stream to read from and write to
        EEPROM.commit(); // does nothing if nothing to do
        alreadyConnected = true;
#ifdef DEBUG
        Serial.println(F("Got Connection "));
#endif
        setLedFlashType(SLOW_LED_FLASH);
      }

      uint8_t cmd = parser.parse(); // parse incoming data from connection
      // parser returns non-zero when a pfod command is fully parsed
      if (cmd != 0) { // have parsed a complete msg { to }
        uint8_t* pfodFirstArg = parser.getFirstArg(); // may point to \0 if no arguments in this msg.
        pfod_MAYBE_UNUSED(pfodFirstArg); // may not be used, just suppress warning
        long pfodLongRtn; // used for parsing long return arguments, if any
        pfod_MAYBE_UNUSED(pfodLongRtn); // may not be used, just suppress warning
        if ('.' == cmd) {
          // pfodApp has connected and sent {.} , it is asking for the main menu
          if (!parser.isRefresh()) {
            sendMainMenu(); // send back the menu designed
          } else {
            sendMainMenuUpdate(); // menu is cached just send update
          }

        } else if ('A' == cmd) {
          // in the main Menu of Menu_1
          char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
          if (parser.isPress()) {
            // send repeat
            Serial.println('>');
          } else if (parser.isUp()) {
            // send stop repeat
            Serial.println('.');
          } else {
            // anything else
            // send button cmd to IR TX
            Serial.print('A');
            Serial.println(dwgCmd);
          }
          // now send response back to pfodApp via WiFi
          parser.print(F("{}")); // finished
          // always send back a response or pfodApp will timeout

        } else if ('B' == cmd) {
          // in the main Menu of Menu_1
          char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
          if (parser.isPress()) {
            // send repeat
            Serial.println('>');
          } else if (parser.isUp()) {
            // send stop repeat
            Serial.println('.');
          } else {
            // anything else
            // send button cmd to IR TX
            Serial.print('B');
            Serial.println(dwgCmd);
          }
          // now send response back to pfodApp via WiFi
          parser.print(F("{}")); // finished
          // always send back a response or pfodApp will timeout

        } else if ('a' == cmd) {
          // get starting offset
          if (!parser.isRefresh()) {
            parser.parseLong(pfodFirstArg, &pfodLongRtn); // get starting offset
            long startingOffset = pfodLongRtn;
            if (startingOffset == 0) {
              sendDrawing_a0();
            } else if (startingOffset == 1) {
              sendDrawing_a1();
            } else if (startingOffset == 2) {
              sendDrawing_a2();
            } else if (startingOffset == 3) {
              sendDrawing_a3();
            } else if (startingOffset == 4) {
              sendDrawing_a4();
            } else if (startingOffset == 5) {
              sendDrawing_a5();
            } else {
              parser.print(F("{}")); // finished
            }

          } else {
            sendDrawingUpdates_a();
          }

        } else if ('b' == cmd) {
          // get starting offset
          if (!parser.isRefresh()) {
            parser.parseLong(pfodFirstArg, &pfodLongRtn); // get starting offset
            long startingOffset = pfodLongRtn;
            if (startingOffset == 0) {
              sendDrawing_b0();
            } else if (startingOffset == 1) {
              sendDrawing_b1();
            } else if (startingOffset == 2) {
              sendDrawing_b2();
            } else if (startingOffset == 3) {
              sendDrawing_b3();
            } else if (startingOffset == 4) {
              sendDrawing_b4();
            } else if (startingOffset == 5) {
              sendDrawing_b5();
            } else if (startingOffset == 6) {
              sendDrawing_b6();
            } else {
              parser.print(F("{}")); // finished
            }

          } else {
            sendDrawingUpdates_b();
          }
        } else {
          parser.print(F("{}")); // finished
        }
      }
    }
  }
  //  <<<<<<<<<<<  Your other loop() code goes here

}

void closeConnection(Stream *io) {
#ifdef DEBUG
  Serial.print(F("closeConnection "));
#endif

  // add any special code here to force connection to be dropped
  parser.closeConnection(); // NULLs io stream
  alreadyConnected = false;
  bufferedClient.stop(); // clears client reference
  WiFiClient::stopAll(); // cleans up memory
  setLedFlashType(NO_LED_FLASH);
  client = server.available(); // evaluates to false if no connection
}

int swap01(int in) {
  return (in == 0) ? 1 : 0;
}

void sendMainMenu() {
  // !! Remember to change the parser version string
  // every time you edit this method
  parser.print(F("{,")); // start a Menu screen pfod message
  // send menu background, format, prompt, refresh and version
  parser.print(F("~`0"));
  parser.sendVersion(); // send the menu version
  // send menu items
  parser.print(F("|+A~a"));
  parser.print(F("|+B~b"));
  parser.print(F("}")); // close pfod message

}

void sendMainMenuUpdate() {
  parser.print(F("{;")); // start an Update Menu pfod message
  // send menu items
  parser.print(F("|A"));
  parser.print(F("|B"));
  parser.print(F("}")); // close pfod message
  // ============ end of menu ===========

}

void sendDrawing_a0() {
  dwgs.start(50, 57, dwgs.WHITE, true); // true means more dwg to come
  parser.sendVersion(); // send the parser version to cache this image
  dwgs.pushZero(0, 0);

  drawLeftTriangle(15.5, 12);
  dwgs.pushZero(15.5, 12);
  dwgs.circle().color(dwgs.GREY).radius(3).send();
  // this button repeats
  dwgs.touchZone().cmd(pvr_leftCmd_A).filter(BUTTON_REPEATS_FILTER).centered().size(6, 6).send();
  dwgs.touchAction().cmd(pvr_leftCmd_A)
  .action(dwgs.circle().filled().color(dwgs.GREY).radius(3)).send();
  dwgs.popZero();

  drawRightTriangle(34.5, 12);
  dwgs.pushZero(34.5, 12);
  dwgs.circle().color(dwgs.GREY).radius(3).send();
  dwgs.touchZone().cmd(pvr_rightCmd_A).filter(BUTTON_REPEATS_FILTER).centered().size(6, 6).send();
  dwgs.touchAction().cmd(pvr_rightCmd_A)
  .action(dwgs.circle().filled().color(dwgs.GREY).radius(3)).send();
  dwgs.popZero();

  drawUpTriangle(25, 3);
  dwgs.pushZero(25, 3);
  dwgs.circle().color(dwgs.GREY).radius(3).send();
  dwgs.touchZone().cmd(pvr_upCmd_A).filter(BUTTON_REPEATS_FILTER).centered().size(6, 6).send();
  dwgs.touchAction().cmd(pvr_upCmd_A)
  .action(dwgs.circle().filled().color(dwgs.GREY).radius(3)).send();
  dwgs.popZero();

  drawDownTriangle(25, 21);
  dwgs.pushZero(25, 21);
  dwgs.circle().color(dwgs.GREY).radius(3).send();
  dwgs.touchZone().cmd(pvr_downCmd_A).filter(BUTTON_REPEATS_FILTER).centered().size(6, 6).send();
  dwgs.touchAction().cmd(pvr_downCmd_A)
  .action(dwgs.circle().filled().color(dwgs.GREY).radius(3)).send();
  dwgs.popZero();


  dwgs.pushZero(25, 12);
  dwgs.circle().radius(3.5).send();
  dwgs.label().text("OK").bold().send();
  dwgs.touchZone().cmd(pvr_okCmd_A).centered().size(7, 7).send();
  dwgs.touchAction().cmd(pvr_okCmd_A).action(dwgs.circle().filled().radius(3.5).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.popZero();
  dwgs.end();
}

void sendDrawing_a1() {
  dwgs.startUpdate(true);
  dwgs.pushZero(0, 3);

  dwgs.pushZero(7, 0);
  dwgs.circle().radius(2.8).send();
  dwgs.circle().color(dwgs.RED).filled().radius(2.5).send();
  dwgs.circle().color(dwgs.WHITE).radius(1.5).send();
  dwgs.line().color(dwgs.WHITE).offset(0, -2).size(0, 2.3).send();
  dwgs.touchZone().cmd(tv_powerCmd_A).centered().size(5, 5).send();
  dwgs.touchAction().cmd(tv_powerCmd_A).action(dwgs.circle().filled().radius(2.5).color(dwgs.GREY)).send();
  dwgs.pushZero(5, 0);
  dwgs.label().text("TV").bold().send();
  dwgs.popZero();
  dwgs.popZero();

  drawMute(40, 0);
  dwgs.pushZero(40, 0);
  dwgs.pushZero(-0.5, 0);
  dwgs.rectangle().rounded().centered().size(8, 5).color(dwgs.GREY).send();
  dwgs.pushZero(6.5, 0);
  dwgs.label().text("TV").bold().send();
  dwgs.popZero();
  dwgs.touchZone().cmd(tv_muteCmd_A).centered().size(8, 5).send();
  dwgs.touchAction().cmd(tv_muteCmd_A)
  .action(dwgs.rectangle().filled().rounded().centered().size(8, 5).color(dwgs.GREY)).send();
  dwgs.popZero();
  dwgs.popZero();

  dwgs.popZero();

  dwgs.pushZero(0, 21);

  dwgs.pushZero(10, 0);
  dwgs.rectangle().rounded().centered().size(11.3, 4.3).send();
  dwgs.rectangle().rounded().centered().size(12.3, 5.3).send();
  dwgs.label().text("Exit").bold().send();
  dwgs.touchZone().cmd(pvr_exitCmd_A).centered().size(12, 5).send();
  dwgs.touchAction().cmd(pvr_exitCmd_A)
  .action(dwgs.rectangle().filled().rounded().centered().size(12, 5).color(dwgs.GREY)).send();
  dwgs.popZero();


  dwgs.pushZero(40, 0);
  dwgs.rectangle().rounded().centered().size(15, 5).color(dwgs.GREY).send();
  dwgs.label().text("Recordings").send();
  dwgs.touchZone().cmd(pvr_recordingsCmd_A).centered().size(15, 5).send();
  dwgs.touchAction().cmd(pvr_recordingsCmd_A)
  .action(dwgs.rectangle().filled().rounded().centered().size(15, 5).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.popZero();
  dwgs.end();
}

void sendDrawing_a2() {
  dwgs.startUpdate(true);
  // Play stop pause FF FR
  dwgs.pushZero(0, 30);

  drawPlay(7, 0);
  draw65square(7, 0, pvr_playCmd_A, NULL);

  drawStop(16, 0);
  draw65square(16, 0, pvr_stopCmd_A, NULL);

  drawPause(25, 0);
  draw65square(25, 0, pvr_pauseCmd_A, NULL);

  drawFastForward(34, 0);
  draw65square(34, 0, pvr_fastForwardCmd_A, NULL);

  drawFastReverse(43, 0);
  draw65square(43, 0, pvr_fastReverseCmd_A, NULL);

  dwgs.popZero();
  dwgs.end();
}

void sendDrawing_a3() {
  dwgs.startUpdate(true);

  dwgs.pushZero(0, 38);

  dwgs.pushZero(7, 0);
  dwgs.circle().radius(2.8).send();
  dwgs.circle().color(dwgs.RED).filled().radius(2.5).send();
  dwgs.touchZone().cmd(pvr_redCmd_A).centered().size(5, 5).send();
  dwgs.touchAction().cmd(pvr_redCmd_A).action(dwgs.circle().filled().radius(2.5).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.pushZero(16, 0);
  dwgs.circle().radius(2.8).send();
  dwgs.circle().color(dwgs.GREEN).filled().radius(2.5).send();
  dwgs.touchZone().cmd(pvr_greenCmd_A).centered().size(5, 5).send();
  dwgs.touchAction().cmd(pvr_greenCmd_A).action(dwgs.circle().filled().radius(2.5).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.pushZero(25, 0);
  dwgs.circle().radius(2.8).send();
  dwgs.circle().color(dwgs.YELLOW).filled().radius(2.5).send();
  dwgs.touchZone().cmd(pvr_yellowCmd_A).centered().size(5, 5).send();
  dwgs.touchAction().cmd(pvr_yellowCmd_A).action(dwgs.circle().filled().radius(2.5).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.pushZero(34, 0);
  dwgs.circle().radius(2.8).send();
  dwgs.circle().color(dwgs.BLUE).filled().radius(2.5).send();
  dwgs.touchZone().cmd(pvr_blueCmd_A).centered().size(5, 5).send();
  dwgs.touchAction().cmd(pvr_blueCmd_A).action(dwgs.circle().filled().radius(2.5).color(dwgs.GREY)).send();
  dwgs.popZero();

  draw65square(43, 0, tv_avCmd_A, "<b>AV");

  dwgs.popZero();
  dwgs.end();
}

void sendDrawing_a4() {
  dwgs.startUpdate(true);
  dwgs.pushZero(0, 46);

  //draw65square(7, 0, pvr_guideCmd_A, "<b><-1>Guide");
  dwgs.pushZero(7, 0);
  dwgs.label().text("Guide").bold().fontSize(-1).send();
  dwgs.rectangle().rounded().centered().size(8, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(pvr_guideCmd_A).centered().size(8, 5).send();
  dwgs.touchAction().cmd(pvr_guideCmd_A)
  .action(dwgs.rectangle().filled().rounded().centered().size(8, 5).color(dwgs.GREY)).send();
  dwgs.popZero();

  drawLeftTriangle(19, 0);
  //draw65square(19, 0, pvr_VolDw_A, NULL);
  dwgs.pushZero(19, 0);
  dwgs.rectangle().rounded().centered().size(6, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(pvr_VolDw_A).filter(BUTTON_REPEATS_FILTER).centered().size(6, 5).send();
  dwgs.touchAction().cmd(pvr_VolDw_A)
  .action(dwgs.rectangle().filled().rounded().centered().size(6, 5).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.pushZero(25, 0);
  dwgs.label().text("<+1>VOL<-1>\nPVR").fontSize(-3).bold().send();
  dwgs.popZero();

  drawRightTriangle(31, 0);
  //draw65square(31, 0, pvr_VolUp_A, NULL);
  dwgs.pushZero(31, 0);
  dwgs.rectangle().rounded().centered().size(6, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(pvr_VolUp_A).filter(BUTTON_REPEATS_FILTER).centered().size(6, 5).send();
  dwgs.touchAction().cmd(pvr_VolUp_A)
  .action(dwgs.rectangle().filled().rounded().centered().size(6, 5).color(dwgs.GREY)).send();
  dwgs.popZero();

  draw65square(43, 0, pvr_popUpCmd_A, "<b><-3>Pop\nUp");

  dwgs.popZero();
  dwgs.end();
}

void sendDrawing_a5() {
  dwgs.startUpdate(true);
  dwgs.pushZero(0, 54);

  draw65square(7, 0, pvr_recCmd_A, "<r><b>Rec");

  drawLeftTriangle(19, 0);
  dwgs.pushZero(19, 0);
  dwgs.rectangle().rounded().centered().size(6, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(tv_VolDw_A).filter(BUTTON_REPEATS_FILTER).centered().size(6, 5).send();
  dwgs.touchAction().cmd(tv_VolDw_A)
  .action(dwgs.rectangle().filled().rounded().centered().size(6, 5).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.pushZero(25, 0);
  dwgs.label().text("<+1>VOL<-1>\nTV").fontSize(-3).bold().send();
  dwgs.popZero();

  drawRightTriangle(31, 0);
  dwgs.pushZero(31, 0);
  dwgs.rectangle().rounded().centered().size(6, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(tv_VolUp_A).filter(BUTTON_REPEATS_FILTER).centered().size(6, 5).send();
  dwgs.touchAction().cmd(tv_VolUp_A)
  .action(dwgs.rectangle().filled().rounded().centered().size(6, 5).color(dwgs.GREY)).send();
  dwgs.popZero();

  draw65square(43, 0, pvr_subTitlesCmd_A, "<-3><b>Sub\nTitles");

  dwgs.popZero();
  dwgs.end();
}

void sendDrawingUpdates_a() {
  // not updates here
  // Note: need start/end to redisplay dwg on re-connection
  dwgs.startUpdate();
  dwgs.end();
}

void sendDrawing_b0() {
  dwgs.start(50, 57, dwgs.WHITE, true); // true means more dwg to come
  parser.sendVersion(); // send the parser version to cache this image
  dwgs.pushZero(0, 4);

  drawGoBack(7, 0);
  dwgs.pushZero(7, 0);
  dwgs.rectangle().rounded().centered().size(6, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(pvr_goBackCmd_B).centered().size(6, 5).send();
  dwgs.touchAction().cmd(pvr_goBackCmd_B).action(dwgs.rectangle().filled().rounded().centered().size(6, 5).color(dwgs.GREY))
  .send();
  dwgs.popZero();

  draw65square(16, 0, pvr_listCmd_B, "List");

  dwgs.pushZero(26.5, 0);
  dwgs.label().text("PVR<-1>\nExtras").bold().fontSize(+1).send();
  dwgs.popZero();

  drawMute(40, 0);
  dwgs.pushZero(40, 0);
  dwgs.pushZero(-0.5, 0);
  dwgs.rectangle().rounded().centered().size(8, 5).color(dwgs.GREY).send();
  dwgs.pushZero(7, 0);
  dwgs.label().text("PVR").fontSize(-1).bold().send();
  dwgs.popZero();
  dwgs.touchZone().cmd(pvr_muteCmd_B).centered().size(8, 5).send();
  dwgs.touchAction().cmd(pvr_muteCmd_B)
  .action(dwgs.rectangle().filled().rounded().centered().size(8, 5).color(dwgs.GREY)).send();
  dwgs.popZero();
  dwgs.popZero();

  dwgs.popZero();
  dwgs.end();
}

void sendDrawing_b1() {
  dwgs.startUpdate(true);

  dwgs.pushZero(0, 12);

  drawJumpForward(7, 0);
  draw65square(7, 0, pvr_jumpForwardCmd_B, NULL);

  drawJumpReverse(16, 0);
  draw65square(16, 0, pvr_jumpReverseCmd_B, NULL);

  drawSlowForward(25, 0);
  draw65square(25, 0, pvr_slowForwardCmd_B, NULL);

  dwgs.pushZero(34, 0);
  dwgs.circle().radius(2.8).send();
  dwgs.label().bold().text("i").fontSize(+2).send();
  dwgs.touchZone().cmd(pvr_infoCmd_B).centered().size(5, 5).send();
  dwgs.touchAction().cmd(pvr_infoCmd_B).action(dwgs.circle().filled().radius(2.5).color(dwgs.GREY)).send();
  dwgs.popZero();

  draw65square(43, 0, pvr_favCmd_B, "<b>Fav");
  dwgs.pushZero(43, 0);
  dwgs.rectangle().rounded().centered().size(6, 5).color(dwgs.BLACK).send();
  dwgs.rectangle().rounded().centered().size(7.3, 6.3).color(dwgs.BLACK).send();
  dwgs.popZero();

  dwgs.popZero();
  dwgs.end();
}

void sendDrawing_b2() {
  dwgs.startUpdate(true);
  dwgs.pushZero(0, 20);

  draw65square(7, 0, pvr_1Cmd_B, "1");
  draw65square(16, 0, pvr_2Cmd_B, "2");
  draw65square(25, 0, pvr_3Cmd_B, "3");
  draw65square(34, 0, pvr_4Cmd_B, "4");
  draw65square(43, 0, pvr_5Cmd_B, "5");

  dwgs.popZero();
  dwgs.end();
}

void sendDrawing_b3() {
  dwgs.startUpdate(true);

  dwgs.pushZero(0, 28);

  draw65square(7, 0, pvr_6Cmd_B, "6");
  draw65square(16, 0, pvr_7Cmd_B, "7");
  draw65square(25, 0, pvr_8Cmd_B, "8");
  draw65square(34, 0, pvr_9Cmd_B, "9");
  draw65square(43, 0, pvr_0Cmd_B, "0");

  dwgs.popZero();
  dwgs.end();
}

void sendDrawing_b4() {
  dwgs.startUpdate(true);
  dwgs.pushZero(0, 36);

  drawLeftTriangle(7, 0);
  draw65square(7, 0, pvr_leftCmd_B, NULL);
  dwgs.pushZero(7, 0);
  // this button repeats
  dwgs.touchZone().cmd(pvr_leftCmd_B).filter(BUTTON_REPEATS_FILTER).centered().size(6, 5).send();
  dwgs.touchAction().cmd(pvr_leftCmd_B)
  .action(dwgs.rectangle().centered().rounded().filled().color(dwgs.GREY).size(6, 5)).send();
  dwgs.popZero();

  drawRightTriangle(16, 0);
  draw65square(16, 0, pvr_rightCmd_B, NULL);
  dwgs.pushZero(16, 0);
  // this button repeats
  dwgs.touchZone().cmd(pvr_rightCmd_B).filter(BUTTON_REPEATS_FILTER).centered().size(6, 5).send();
  dwgs.touchAction().cmd(pvr_rightCmd_B)
  .action(dwgs.rectangle().centered().rounded().filled().color(dwgs.GREY).size(6, 5)).send();
  dwgs.popZero();

  drawUpTriangle(25, 0);
  draw65square(25, 0, pvr_upCmd_B, NULL);
  dwgs.pushZero(25, 0);
  // this button repeats
  dwgs.touchZone().cmd(pvr_upCmd_B).filter(BUTTON_REPEATS_FILTER).centered().size(6, 5).send();
  dwgs.touchAction().cmd(pvr_upCmd_B)
  .action(dwgs.rectangle().centered().rounded().filled().color(dwgs.GREY).size(6, 5)).send();
  dwgs.popZero();

  drawDownTriangle(34, 0);
  draw65square(34, 0, pvr_downCmd_B, NULL);
  dwgs.pushZero(34, 0);
  // this button repeats
  dwgs.touchZone().cmd(pvr_downCmd_B).filter(BUTTON_REPEATS_FILTER).centered().size(6, 5).send();
  dwgs.touchAction().cmd(pvr_downCmd_B)
  .action(dwgs.rectangle().centered().rounded().filled().color(dwgs.GREY).size(6, 5)).send();
  dwgs.popZero();

  dwgs.pushZero(43, 0);
  dwgs.circle().radius(3.5).send();
  dwgs.label().text("OK").bold().send();
  dwgs.touchZone().cmd(pvr_okCmd_B).centered().size(7, 7).send();
  dwgs.touchAction().cmd(pvr_okCmd_B).action(dwgs.circle().filled().radius(3).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.popZero();
  dwgs.end();
}

void sendDrawing_b5() {
  dwgs.startUpdate(true);
  dwgs.pushZero(0, 44);

  dwgs.pushZero(7, 0);
  dwgs.circle().radius(2.8).send();
  dwgs.circle().color(dwgs.RED).filled().radius(2.5).send();
  dwgs.circle().color(dwgs.WHITE).radius(1.5).send();
  dwgs.line().color(dwgs.WHITE).offset(0, -2).size(0, 2.3).send();
  dwgs.touchZone().cmd(pvr_powerCmd_B).centered().size(5, 5).send();
  dwgs.touchAction().cmd(pvr_powerCmd_B).action(dwgs.circle().filled().radius(2.5).color(dwgs.GREY)).send();
  dwgs.popZero();

  draw65square(14, 0, pvr_tvoutCmd_B, "<-4><b>TV\nOUT");
  draw65square(21, 0, pvr_liveCmd_B, "<-4><b>LIVE");

  dwgs.pushZero(30, 0);
  dwgs.label().text("Guide").bold().fontSize(-1).send();
  dwgs.rectangle().rounded().centered().size(8, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(pvr_guideCmd_B).centered().size(8, 5).send();
  dwgs.touchAction().cmd(pvr_guideCmd_B)
  .action(dwgs.rectangle().filled().rounded().centered().size(8, 5).color(dwgs.GREY)).send();
  dwgs.popZero();


  dwgs.pushZero(42, 0);
  dwgs.rectangle().rounded().centered().size(11.3, 4.3).send();
  dwgs.rectangle().rounded().centered().size(12.3, 5.3).send();
  dwgs.label().text("Exit").bold().send();
  dwgs.touchZone().cmd(pvr_exitCmd_B).centered().size(12, 5).send();
  dwgs.touchAction().cmd(pvr_exitCmd_B)
  .action(dwgs.rectangle().filled().rounded().centered().size(12, 5).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.popZero();
  dwgs.end();
}

void sendDrawing_b6() {
  dwgs.startUpdate(true);
  dwgs.pushZero(0, 52);

  drawUpTriangle(7, 0);
  dwgs.pushZero(7, 0);
  dwgs.rectangle().rounded().centered().size(6, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(pvr_ChUpCmd_B).filter(BUTTON_REPEATS_FILTER).centered().size(6, 5).send();
  dwgs.touchAction().cmd(pvr_ChUpCmd_B)
  .action(dwgs.rectangle().filled().rounded().centered().size(6, 5).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.pushZero(14, 0);
  dwgs.label().text("CH").fontSize(+1).bold().send();
  dwgs.popZero();

  drawDownTriangle(21, 0);
  dwgs.pushZero(21, 0);
  dwgs.rectangle().rounded().centered().size(6, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(pvr_ChDnCmd_B).filter(BUTTON_REPEATS_FILTER).centered().size(6, 5).send();
  dwgs.touchAction().cmd(pvr_ChDnCmd_B)
  .action(dwgs.rectangle().filled().rounded().centered().size(6, 5).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.pushZero(30, 0);
  dwgs.label().text("Zoom").fontSize(-1).send();
  dwgs.rectangle().rounded().centered().size(8, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(pvr_zoomCmd_B).centered().size(8, 5).send();
  dwgs.touchAction().cmd(pvr_zoomCmd_B)
  .action(dwgs.rectangle().filled().rounded().centered().size(8, 5).color(dwgs.GREY)).send();
  dwgs.popZero();


  dwgs.pushZero(42, 0);
  dwgs.label().text("SETUP").fontSize(-2).bold().send();
  dwgs.rectangle().rounded().centered().size(12, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(pvr_setupCmd_B).centered().size(12, 5).send();
  dwgs.touchAction().cmd(pvr_setupCmd_B)
  .action(dwgs.rectangle().filled().rounded().centered().size(12, 5).color(dwgs.GREY)).send();
  dwgs.popZero();

  dwgs.popZero();
  dwgs.end();
}

// teletext  0xBE0051AE
// soundtrack 0XBE004986
// sleep 0xBE001BE4

void sendDrawingUpdates_b() {
  // not updates here
  // Note: need start/end to redisplay dwg on re-connection
  dwgs.startUpdate();
  dwgs.end();
}


void drawLeftTriangle(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.line().offset(1, -1).size(0, 2).send();
  dwgs.line().offset(1, -1).size(-2, 1).send();
  dwgs.line().offset(1, 1).size(-2, -1).send();
  dwgs.popZero();
}

void drawRightTriangle(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.line().offset(-1, -1).size(0, 2).send();
  dwgs.line().offset(-1, -1).size(2, 1).send();
  dwgs.line().offset(-1, 1).size(2, -1).send();
  dwgs.popZero();
}

void drawUpTriangle(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.line().offset(-1, 1).size(2, 0).send();
  dwgs.line().offset(-1, 1).size(1, -2).send();
  dwgs.line().offset(1, 1).size(-1, -2).send();
  dwgs.popZero();
}

void drawDownTriangle(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.line().offset(-1, -1).size(2, 0).send();
  dwgs.line().offset(-1, -1).size(1, 2).send();
  dwgs.line().offset(1, -1).size(-1, 2).send();
  dwgs.popZero();
}

void drawPause(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.line().size(0, 2).offset(-0.5, -1).send();
  dwgs.line().size(0, 2).offset(0.5, -1).send();
  dwgs.popZero();
}

void drawPlay(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.line().offset(-1, -1).size(0, 2).send();
  dwgs.line().offset(-1, -1).size(2, 1).send();
  dwgs.line().offset(-1, 1).size(2, -1).send();
  dwgs.popZero();
}

void drawStop(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.rectangle().filled().rounded().size(2, 2).centered().send();
  dwgs.popZero();
}

void drawFastReverse(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.line().offset(1.75, -1).size(-2, 1).send();
  dwgs.line().offset(1.75, 1).size(-2, -1).send();
  dwgs.line().offset(0.25, -1).size(-2, 1).send();
  dwgs.line().offset(0.25, 1).size(-2, -1).send();
  dwgs.popZero();
}

void drawFastForward(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.line().offset(-1.75, -1).size(2, 1).send();
  dwgs.line().offset(-1.75, 1).size(2, -1).send();
  dwgs.line().offset(-0.25, -1).size(2, 1).send();
  dwgs.line().offset(-0.25, 1).size(2, -1).send();
  dwgs.popZero();
}

void drawJumpReverse(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.line().offset(1.75, -1).size(-2, 1).send();
  dwgs.line().offset(1.75, 1).size(-2, -1).send();
  dwgs.line().offset(0.25, -1).size(-2, 1).send();
  dwgs.line().offset(0.25, 1).size(-2, -1).send();
  dwgs.line().offset(-1.85, -1.5).size(0, 3).send();
  dwgs.popZero();
}

void drawJumpForward(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.line().offset(-1.75, -1).size(2, 1).send();
  dwgs.line().offset(-1.75, 1).size(2, -1).send();
  dwgs.line().offset(-0.25, -1).size(2, 1).send();
  dwgs.line().offset(-0.25, 1).size(2, -1).send();
  dwgs.line().offset(1.85, -1.5).size(0, 3).send();
  dwgs.popZero();
}

void drawSlowForward(double c, double r) {
  dwgs.pushZero(c, r);
  dwgs.pushZero(0.75, 0);
  dwgs.line().offset(-1, -1).size(0, 2).send();
  dwgs.line().offset(-1, -1).size(2, 1).send();
  dwgs.line().offset(-1, 1).size(2, -1).send();
  dwgs.rectangle().filled().offset(-2, -1).size(1, 2).send();
  dwgs.popZero();
  dwgs.popZero();
}

void drawGoBack(double c, double r) { 
  dwgs.pushZero(c, r);
  dwgs.circle().radius(1.5).send();
  //dwgs.arc().radius(1.5).filled().start(120).angle(-300).send();
  dwgs.arc().radius(1.7).filled().color(dwgs.WHITE).start(135).angle(35).send();
  dwgs.line().offset(-1.5, 0).size(1.2, 0.6).send();
  dwgs.line().offset(-1.5, 0).size(-0.8, 1).send();
  dwgs.popZero();
}

void drawMute(double c, double r) { 
  dwgs.pushZero(c, r, 0.8);
  dwgs.pushZero(-3, -1);
  dwgs.rectangle().size(1, 2).send();
  dwgs.line().offset(1, 0).size(2, -1.5).send();
  dwgs.line().offset(1, 2).size(2, 1.5).send();
  dwgs.line().offset(3, -1.5).size(0, 5).send();
  dwgs.popZero();
  dwgs.line().offset(-2, -2.5).size(4, 5).send();
  dwgs.line().offset(-2, 2.5).size(4, -5).send();
  dwgs.popZero();
}

void draw65square(double c, double r, char cmd, char* label) {
  dwgs.pushZero(c, r);
  if (label != NULL) {
    dwgs.label().text(label).send();
  }
  dwgs.rectangle().rounded().centered().size(6, 5).color(dwgs.GREY).send();
  dwgs.touchZone().cmd(cmd).centered().size(6, 5).send();
  dwgs.touchAction().cmd(cmd).action(dwgs.rectangle().filled().rounded().centered().size(6, 5).color(dwgs.GREY))
  .send();
  dwgs.popZero();
}


